@@ -15,11 +15,18 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-vcreview specific hooks and operations"""
 
+__docformat__ = "restructuredtext en"
+
+import re
+
 from cubicweb.server import hook
 from cubicweb.selectors import is_instance
 
 IGNORE_FILES = set( ('README', 'series', 'status', '.hgignore', '.hgtags') )
 
+def fire(patch, trname, comment):
+    patch.cw_adapt_to('IWorkflowable').fire_transition(trname, comment)
+
 
 class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
     containercls = list
@@ -46,8 +53,10 @@ class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
             # create a new patch entity if no one found or if found a closed
             # patch and this is not a file deletion
             if patch is None or (not patch.is_active() and entity.__regid__ == 'VersionContent'):
+                revision = entity.rev
                 patch = self.session.create_entity(
-                    'Patch', patch_repository=entity.rev.from_repository)
+                    'Patch', originator=revision.author, branch=revision.branch,
+                    patch_repository=revision.from_repository)
             patch.set_relations(patch_revision=entity)
             # patch file is being removed: mark it on the patch entity if still
             # active
@@ -88,72 +97,59 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
         for rev in self.get_data():
             # search patches among files modified by the revision
             readycandidates = {}
+            pendingcandidates = {}
             rejectcandidates = {}
             for vc in rev.reverse_from_revision:
                 if not vc.patch:
                     continue
                 pstate = vc.patch.cw_adapt_to('IWorkflowable').state
-                if pstate == 'in-progress' and vc.__regid__ == 'VersionContent':
-                    readycandidates[vc.file.path] = vc.patch
+                if vc.__regid__ == 'VersionContent':
+                    if pstate == 'in-progress':
+                        readycandidates[vc.file.path.lower()] = vc.patch
+                    elif pstate == 'pending-review':
+                        pendingcandidates[vc.file.path.lower()] = vc.patch
                 elif (pstate in ('in-progress', 'pending-review', 'deleted') and
                       vc.__regid__ == 'DeletedVersionContent'):
-                    rejectcandidates[vc.file.path] = vc.patch
+                    rejectcandidates[vc.file.path.lower()] = vc.patch
+            interestingwords = set( ('ready', 'applied', 'reject', 'fold') )
+            interestingwords.update(readycandidates)
+            interestingwords.update(pendingcandidates)
+            interestingwords.update(rejectcandidates)
             # search for instruction
             for line in rev.description.splitlines():
-                words = line.strip().split()
-                if not len(words) >= 2:
+                words = [w.lower() for w in re.findall('[-\.\w]+', line.strip())
+                         if w.lower() in interestingwords]
+                if len(words) < 2:
                     continue
-                if words[0] in readycandidates and words[1:4] == ['ready', 'for', 'review']:
-                    patch = readycandidates.pop(words[0])
-                    patch.cw_adapt_to('IWorkflowable').fire_transition(
-                        'ask review', 'review asked by %s in %s' % (
-                            rev.author, rev.changeset or rev.revision))
+                if words[1] == 'ready' and words[0] in readycandidates:
+                    fire(readycandidates.pop(words[0]),
+                         'ask review', u'review asked by %s in %s' % (
+                             rev.author, rev.changeset or rev.revision))
                 elif words[0] == 'reject' and words[1] in rejectcandidates:
                     patch = rejectcandidates.pop(words[1])
-                    patch.cw_adapt_to('IWorkflowable').fire_transition(
-                        'reject', 'rejected by %s in %s' % (
-                            rev.author, rev.changeset or rev.revision))
-                    MarkPatchAsDeletedOp.get_instance(self.session)._container.remove(patch)
+                    fire(patch, 'reject', u'rejected by %s in %s' % (
+                        rev.author, rev.changeset or rev.revision))
+                    self.ensure_wont_be_deleted(patch)
                 elif words[0] == 'fold' and words[1] in rejectcandidates:
                     patch = rejectcandidates.pop(words[1])
-                    patch.cw_adapt_to('IWorkflowable').fire_transition(
-                        'fold', 'folded by %s in %s' % (
-                            rev.author, rev.changeset or rev.revision))
-                    MarkPatchAsDeletedOp.get_instance(self.session)._container.remove(patch)
+                    fire(patch, 'fold', u'folded by %s in %s' % (
+                        rev.author, rev.changeset or rev.revision))
+                    self.ensure_wont_be_deleted(patch)
+                elif words[0] == 'applied' and words[1] in pendingcandidates:
+                    patch = pendingcandidates.pop(words[1])
+                    fire(patch, 'accept', u'accepted by %s' % rev.author)
+                    self.ensure_wont_be_deleted(patch)
+                    # XXX search source repository changeset id
+                    #patch.set_relations(applied_at=rev)
 
 
-class SearchAppliedPatchOp(hook.DataOperationMixIn,
-                           hook.Operation):
-    """search magic word in revision's commit message:
-
-        applied patch <patch path>
-
-    When found, the patch is marked as applied and linked to the revision.
-    """
-    containercls = list
-    def precommit_event(self):
-        for rev in self.get_data():
-            patchrepo = rev.repository.patchrepo
-            # search patches among in-progress/deleted/pending-review
-            candidates = {}
-            for patch in self.session.execute(
-                # order by so that we consider the latest patch with activities
-                # in case of conflict (may have due to search of 'deleted'
-                # status)
-                'Any P ORDERBY P ASC WHERE P patch_revision VC, VC from_revision REV, '
-                'REV from_repository R, R eid %(r)s, P in_state PS, '
-                'PS name IN ("in-progress", "deleted", "pending-review")',
-                {'r': patchrepo.eid}).entities():
-                candidates[patch.patch_name] = patch
-            # search for instruction
-            for line in rev.description.splitlines():
-                words = line.strip().split()
-                if not words:
-                    continue
-                if words[-1] in candidates and words[-3:-1] == ['applied', 'patch']:
-                    patch = candidates.pop(words[-1])
-                    patch.cw_adapt_to('IWorkflowable').fire_transition('accept')
-                    patch.set_relations(applied_at=rev)
+    def ensure_wont_be_deleted(self, patch):
+        op = MarkPatchAsDeletedOp.get_instance(self.session)
+        try:
+            # XXX op.remove_data(patch) once cw 3.10.9 is out
+            op._container.remove(patch)
+        except KeyError:
+            pass
 
 
 class RevisionAdded(hook.Hook):
@@ -166,9 +162,6 @@ class RevisionAdded(hook.Hook):
         if repo.patchrepo_of:
             # patch repository
             SearchPatchStateInstructionOp.get_instance(self._cw).add_data(self.entity)
-        elif repo.patchrepo:
-            # main repository linked to patch repository
-            SearchAppliedPatchOp.get_instance(self._cw).add_data(self.entity)
 
 
 # use single last operation to be executed after all vcreview related operations
@@ -184,5 +177,6 @@ class MarkPatchAsDeletedOp(hook.DataOperationMixIn, hook.SingleLastOperation):
 
 
 from cubes.nosylist import hooks as nosylist
-nosylist.S_RELS |= set(('has_activity',))
-nosylist.O_RELS |= set(('patch_repository',))
+nosylist.S_RELS |= set(('has_activity', 'patch_revision'))
+# XXX should add 'comments' relation in set below
+nosylist.O_RELS |= set(('patch_repository', 'point_of'))
