@@ -150,15 +150,19 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
         # fake revision from the main repo of our patch repo
         revision = mock_object(repository=self.vcsrepo,
                                description='applied patch patch3.diff',
-                               eid=patch3_2.revisions[-1].rev.eid)
+                               author='syt',
+                               eid=patch3_2.revisions[-1].rev.eid,
+                               reverse_from_revision=[
+                                   self.session.entity_from_eid(patch3_2.revisions[-2].eid)])
         # hi-hack vcsrepo._cw to avoid connections pool pb
         self.vcsrepo._cw = self.session
-        op = hooks.SearchAppliedPatchOp.get_instance(self.session)
+        self.session.set_pool()
+        op = hooks.SearchPatchStateInstructionOp.get_instance(self.session)
         op.add_data(revision)
         self.commit() # will process our fake operation
         patch3_2.clear_all_caches()
         self.assertEqual(patch3_2.cw_adapt_to('IWorkflowable').state, 'applied')
-        self.assertEqual(patch3_2.applied_at[0].eid, patch3_2.revisions[-1].rev.eid)
+        #self.assertEqual(patch3_2.applied_at[0].eid, patch3_2.revisions[-1].rev.eid) XXX
 
         self.assertEqual(len(MAILBOX), 1, MAILBOX)
         self.assertEqual(MAILBOX[0].subject, '[%s] patch %s is now in state "applied"'
@@ -168,7 +172,7 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
 admin changed status from <pending-review> to <applied> for entity
 'patch3.diff'
 
-
+accepted by syt
 
 remaining tasks:
 
